/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: GetReplayFilenamesCommand.as 9477 2011-12-13 15:07:43Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.command {
import com.adobe.cairngorm.commands.ICommand;
import com.adobe.cairngorm.control.CairngormEvent;
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.rpc.IResponder;
import mx.rpc.events.FaultEvent;
import mx.rpc.events.ResultEvent;

import org.ow2.jasmine.monitoring.eos.common.controls.ServerSideException;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;

import org.ow2.jasmine.monitoring.eos.monitoring.event.MonitoringEvent;
import org.ow2.jasmine.monitoring.eos.monitoring.model.MonitoringModelLocator;
import org.ow2.jasmine.monitoring.eos.monitoring.vo.ReplayFileName;

/**
 * @author Philippe Durieux
 */
[Event(name="serverSideException", type="org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent")]
public class GetReplayFilenamesCommand implements ICommand, IResponder {
    private var model:MonitoringModelLocator = MonitoringModelLocator.getInstance();

    public function GetReplayFilenamesCommand() {
    }

    /**
     * Send the event to the java side, using the business layer of the pattern
     */
    public function execute(event:CairngormEvent):void {
        model.getDelegate().responder = this;
        var ev:MonitoringEvent = event as MonitoringEvent;
        model.getDelegate().getReplayFilenames(ev.args, ev.loadFile);
    }

    /**
     * Get the result of the java side. this method is called on each event from
     * Java.
     */
    public function result(event:Object):void {
        var files:ArrayCollection = (((event as ResultEvent).result) as ArrayCollection);
        if (files != null) {
            // a simple "model.replayFileNames = files" will not work because items
            // will not be types as ReplayFileName, and a generated object will be used instead.
            var tmp:ArrayCollection = new ArrayCollection();
            for each (var f:ReplayFileName in files) {
                tmp.addItem(f);
            }
            model.replayFileNames = tmp;
        }
    }

    /**
     * Handle fault messages
     */
    public function fault(info:Object):void {
        // Retrieve the fault event
        var faultEvent:FaultEvent = FaultEvent(info);

        // Tell the view and let it handle this
        var exc:ServerSideException = new ServerSideException("Error while getting file list", "The operation could not be performed." + "\n" + faultEvent.fault.faultCode + "\n" + faultEvent.fault.faultString, faultEvent.fault.getStackTrace());
        var event:ServerSideExceptionEvent = new ServerSideExceptionEvent("serverSideException", exc);
        CairngormEventDispatcher.getInstance().dispatchEvent(event);
    }

}
}
