/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:Controller.as 3911 2009-06-10 13:27:29Z merisiec $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.eos.monitoring.control {
import com.adobe.cairngorm.control.FrontController;

import org.ow2.jasmine.monitoring.eos.monitoring.command.*;
import org.ow2.jasmine.monitoring.eos.monitoring.model.*;
import org.ow2.jasmine.monitoring.eos.monitoring.event.MonitoringEvent;

/**
 * Bind a type of command to an action.
 *
 * @author Guillaume Renault
 */
public class Controller extends FrontController {

    /**
     * Create an instance of the controller. Commands are initialized here.
     */
    public function Controller() {
        initialiseCommands();
    }

    /**
     * Add all of the commands to the pool of commands.
     */
    public function initialiseCommands():void {
        var model:MonitoringModelLocator = MonitoringModelLocator.getInstance();
        this.addCommand(MonitoringEvent.LOAD_CONF_EVENT + model.componentID, LoadConfigFileCommand);
        this.addCommand(MonitoringEvent.SAVE_CONF_EVENT + model.componentID, SaveConfigFileCommand);
        this.addCommand(MonitoringEvent.REMOVE_CONF_EVENT + model.componentID, RemoveConfigFileCommand);
        this.addCommand(MonitoringEvent.RENAME_CONF_EVENT + model.componentID, RenameConfigFileCommand);
        this.addCommand(MonitoringEvent.CREATE_DASHBOARD + model.componentID, CreateDashboardCommand);
        this.addCommand(MonitoringEvent.REMOVE_DASHBOARD + model.componentID, RemoveDashboardCommand);
        this.addCommand(MonitoringEvent.START_MONITORING + model.componentID, StartMonitoringCommand);
        this.addCommand(MonitoringEvent.STOP_MONITORING + model.componentID, StopMonitoringCommand);
        this.addCommand(MonitoringEvent.START_FILE_REPLAY + model.componentID, StartFileReplayCommand);
        this.addCommand(MonitoringEvent.START_DATABASE_REPLAY + model.componentID, StartDatabaseReplayCommand);
        this.addCommand(MonitoringEvent.GET_LIVE_RESULT + model.componentID, GetLiveResultCommand);
        this.addCommand(MonitoringEvent.GET_REPLAY_FILENAMES_EVENT + model.componentID, GetReplayFilenamesCommand);
        this.addCommand(MonitoringEvent.GET_CONFIG_FILENAMES_EVENT + model.componentID, GetConfigFilenamesCommand);
        this.addCommand(MonitoringEvent.GET_DATABASE_LIMITS + model.componentID, GetDatabaseLimitsCommand);
    }

}
}
