/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MonitoringEvent.as 9538 2012-01-05 09:49:15Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.event {
import com.adobe.cairngorm.control.CairngormEvent;

import flash.events.Event;

import mx.collections.ArrayCollection;

import org.ow2.jasmine.monitoring.eos.monitoring.vo.ConfigFile;

/**
 * @author Guillaume Renault
 */
public class MonitoringEvent extends CairngormEvent {

    // ////////////////////////////////////////////////////////////////////////
    // Event types
    // ////////////////////////////////////////////////////////////////////////
    public static const LOAD_CONF_EVENT:String = "Load configuration file";
    public static const SAVE_CONF_EVENT:String = "Save configuration file";
    public static const REMOVE_CONF_EVENT:String = "Remove configuration file";
    public static const RENAME_CONF_EVENT:String = "Rename configuration file";
    public static const CREATE_DASHBOARD:String = "Create Dashboard";
    public static const REMOVE_DASHBOARD:String = "Remove Dashboard";
    public static const GET_LIVE_RESULT:String = "Get Live Result";
    public static const START_MONITORING:String = "Start Real Time Monitoring";
    public static const STOP_MONITORING:String = "Stop Real Time Monitoring";
    public static const START_FILE_REPLAY:String = "Start File Replay";
    public static const START_DATABASE_REPLAY:String = "Start Database Replay";
    public static const GET_REPLAY_FILENAMES_EVENT:String = "Get Replay Filenames";
    public static const GET_CONFIG_FILENAMES_EVENT:String = "Get Configuration Filenames";
    public static const GET_DATABASE_LIMITS:String = "Get Database Limits";

    /**
     * Configuration file to load.
     */
    private var _loadFile:String = null;

    /**
     * Configuration file to load.
     */
    private var _configFile:ConfigFile = null;

    /**
     * Columns to get when loading a log file.
     */
    private var _headers:String = null;

    /**
     * Series to look at when loading datas
     */
    private var _series:ArrayCollection;

    /**
     * date limits for replay
     */
    private var _from:Date;
    private var _to:Date;

    /**
     * ident of dashboard
     */
    private var _ident:int;

    /**
     * arguments to define what will be selected from database
     */
    private var _arguments:String = null;

    public function MonitoringEvent(type:String) {
        super(type);
    }

    override public function clone():Event {
        return new MonitoringEvent(this.type);
    }

    public function set loadFile(file:String):void {
        this._loadFile = file;
    }

    public function get loadFile():String {
        return this._loadFile;
    }

    public function set configFile(file:ConfigFile):void {
        this._configFile = file;
    }

    public function get configFile():ConfigFile {
        return this._configFile;
    }

    public function set header(head:String):void {
        this._headers = head;
    }

    public function get header():String {
        return this._headers;
    }

    public function set args(args:String):void {
        this._arguments = args;
    }

    public function get args():String {
        return this._arguments;
    }

    public function set series(series:ArrayCollection):void {
        this._series = series;
    }

    public function get series():ArrayCollection {
        return this._series;
    }

    public function get from():Date {
        return this._from;
    }

    public function set from(from:Date):void {
        this._from = from;
    }

    public function get to():Date {
        return this._to;
    }

    public function set to(to:Date):void {
        this._to = to;
    }

    public function get ident():int {
        return this._ident;
    }

    public function set ident(ident:int):void {
        this._ident = ident;
    }
}
}
