/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MonitoringModelLocator.as 9538 2012-01-05 09:49:15Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.model {
import com.adobe.cairngorm.model.ModelLocator;

import mx.collections.ArrayCollection;
import mx.controls.Alert;
import mx.rpc.events.FaultEvent;
import mx.utils.UIDUtil;

import org.ow2.jasmine.monitoring.eos.monitoring.business.MyDelegate;
import org.ow2.jasmine.monitoring.eos.monitoring.view.MonitoringClass;

/**
 * @author Guillaume Renault
 */
[Bindable]
public class MonitoringModelLocator implements ModelLocator {

    /**
     * Reference on MonitoringClass, usable by any other module.
     */
    public var util:MonitoringClass;

    /**
     * The unique ID of this component
     *
     * @internal
     * Used to prevent a Cairngorm issue: when a command event is dispatched,
     * every controller that registered this event type receives it, even if
     * located in another module. To prevent this from happening and triggering
     * multiple severe unexpected concurrence bugs, each event dispatched is
     * postfixed with this unique ID.
     */
    public var componentID:String = UIDUtil.createUID();

    /**
     * Constructor
     */
    public function MonitoringModelLocator() {
        super();

        if (model != null) {
            throw new Error("Only one LaunchCommandModellocator has to be set");
        }
    }

    /**
     * Unique instance of this locator.
     */
    private static var model:MonitoringModelLocator = null;

    public static function getInstance():MonitoringModelLocator {
        if (MonitoringModelLocator.model == null) {
            MonitoringModelLocator.model = new MonitoringModelLocator();
        }

        return MonitoringModelLocator.model;
    }

    /**
     * Ident of the last created dashboard.
     */
    private var _dbIdent:int = -2;

    public function set dbIdent(id:int):void {
        this._dbIdent = id;
    }

    public function get dbIdent():int {
        return this._dbIdent;
    }

    /**
     * String form of the configuration file.
     */
    private var _strconf:String;

    public function set strconf(conf:String):void {
        this._strconf = conf;
    }

    public function get strconf():String {
        return this._strconf;
    }

    /**
     * An ArrayCollection of EventData
     */
    private var _providers:ArrayCollection = null;

    public function set providers(prov:ArrayCollection):void {
        this._providers = prov;
    }

    public function get providers():ArrayCollection {
        return this._providers;
    }

    /**
     * Error returned fromp server
     */
    private var _error:FaultEvent = null;

    public function set onError(error:FaultEvent):void {
        this._error = error;
    }

    public function get onError():FaultEvent {
        return this._error;
    }

    /**
     * Array of ConfigFile
     */
    private var _configFiles:ArrayCollection = new ArrayCollection();

    public function get configFiles():ArrayCollection {
        return _configFiles;
    }

    public function set configFiles(value:ArrayCollection):void {
        this._configFiles = value;
    }

    /**
     * Array of ReplayFileName
     */
    private var _replayFileNames:ArrayCollection = new ArrayCollection();

    public function get replayFileNames():ArrayCollection {
        return _replayFileNames;
    }

    public function set replayFileNames(value:ArrayCollection):void {
        this._replayFileNames = value;
    }

    /**
     * Database Start Timestamp
     */
    private var _databaseStartTime:Number;

    public function get databaseStartTime():Number {
        return _databaseStartTime;
    }

    public function set databaseStartTime(value:Number):void {
        _databaseStartTime = value;
    }

    /**
     * unique instance of delegate
     */
    private var _delegate:MyDelegate = null;

    public function getDelegate():MyDelegate {
        if (_delegate == null) {
            _delegate = new MyDelegate();
        }
        return _delegate;
    }

}
}
