/**
 * JASMINe
 * Copyright (C) 2011 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.view {
import com.adobe.cairngorm.control.CairngormEventDispatcher;

import flash.net.FileFilter;
import flash.utils.Dictionary;

import mx.collections.ArrayCollection;
import mx.containers.Canvas;

import mx.containers.Form;
import mx.containers.FormHeading;
import mx.containers.FormItem;
import mx.containers.TabNavigator;
import mx.containers.TitleWindow;
import mx.containers.VBox;
import mx.controls.Alert;
import mx.controls.DataGrid;
import mx.controls.ProgressBar;
import mx.controls.TextInput;
import mx.events.CloseEvent;
import mx.managers.CursorManager;
import mx.managers.PopUpManager;

import mx.resources.ResourceManager;
import mx.rpc.events.FaultEvent;

import org.ow2.jasmine.monitoring.eos.common.events.LoadEvent;
import org.ow2.jasmine.monitoring.eos.common.events.ServerSideExceptionEvent;
import org.ow2.jasmine.monitoring.eos.common.view.LoadWindow;
import org.ow2.jasmine.monitoring.eos.monitoring.event.MonitoringEvent;
import org.ow2.jasmine.monitoring.eos.monitoring.model.MonitoringModelLocator;
import org.ow2.jasmine.monitoring.eos.monitoring.vo.ConfigFile;

/**
 * Action Script part of MonitoringView
 */
public class MonitoringClass extends Canvas {

    private var dispatcher:CairngormEventDispatcher = CairngormEventDispatcher.getInstance();

    // popup for editor
    private var editConfigWindow:EditConfigWindow = null;

    // Name of the current config file just loaded.
    // Needed for save or edit operations
    public var configFileName:String;

    // params found in the configuration file
    public var params:ArrayCollection = null;

    // For each param, give its text field with the chosen value.
    private var paramValues:Dictionary = new Dictionary();

    // Config file splitted in an Array of Strings
    // Each item is either plain text or param name.
    public var fileTokens:ArrayCollection = null;

    // Config File, with params replaced by their value
    [Bindable]
    public var strConfigFile:String = "";

    private var applyNow:Boolean = false;

    [Bindable]
    protected var model:MonitoringModelLocator = MonitoringModelLocator.getInstance();

    // -------------------------------------------------------------------
    // These variables are defined in the mxml part.
    // -------------------------------------------------------------------

    [Bindable]
    public var listFiles:DataGrid;

    [Bindable]
    public var formparam:Form;

    [Bindable]
    public var tabNavigator:TabNavigator;

    [Bindable]
    public var configWindow:VBox;

    [Bindable]
    public var fname:TextInput;

    [Bindable]
    public var nfname:TextInput;

    // -------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------

    public function MonitoringClass() {
    }

    public function initComponent():void {
        model.util = this; // used to retrive this from every module

        // Listen to server side events
        dispatcher.addEventListener(ServerSideExceptionEvent.SERVER_SIDE_EXCEPTION, ServerSideExceptionEvent.show);

        // Get the current list of configuration files on the server
        getFileList();
        this.currentState = "pickone";
    }

    // -------------------------------------------------------------------
    // Setters triggered by bindings
    // -------------------------------------------------------------------

    /**
     * A new config file has been read from server
     * This is triggered by the binding when model has changed,
     * usually when user has selected a config file in the list.
     * @param file String form of the config file
     */
    public function set strconf(file:String):void {
        // Look for params the config file
        lookForParams(file);
        CursorManager.removeBusyCursor();
        if (this.params != null) {
            this.currentState = "params";
        }
        else {
            this.strConfigFile = this.fileTokens.source[0];
            this.currentState = "selected";
        }
    }

    /**
     * Error from Server
     */
    public function set error(event:FaultEvent):void {
        Alert.show(event.toString(), locale("error_occured"));
    }

    // -------------------------------------------------------------------
    // State changes
    // -------------------------------------------------------------------

    /**
     * Enter state "Pickone"
     */
    public function enterPickone():void {
        // reset all
        this.fileTokens = null;
        this.strConfigFile = "";
        this.configFileName = null;
        this.params = null;
        this.paramValues = new Dictionary();
        this.listFiles.selectedIndex = -1;
    }

    /**
     * Enter state "params"
     * params have been found in the file.
     * Populate the Form with each param found.
     */
    public function enterParams():void {
        // Dynamically build a Form to get the params
        this.formparam.removeAllChildren();

        var first:Boolean = true;
        for each (var param:String in this.params) {
            var item:FormItem = new FormItem();
            item.label = param;
            var input:TextInput = new TextInput();
            input.text = "@" + param + "@";
            item.addChild(input);
            paramValues[param] = input;
            this.formparam.addChild(item);
            if (first) {
                // Set focus on first param
                input.setFocus();
                input.drawFocus(true);
                first = false;
            }
        }
    }

    /**
     * Enter state "selected"
     * File has been selected, and params have been replaced.
     * ready to apply/save/rename/edit/remove this file.
     */
    public function enterSelected():void {
        // Sometimes, we didn't go in "set strconf"
        CursorManager.removeBusyCursor();
        if (applyNow) {
            applyNow = false;
            applyConfig();
        }
    }

    /**
     * Enter state "SaveFile"
     */
    public function enterSaveFile():void {
        focusManager.setFocus(fname);
    }

    /**
     * Enter state "RenameFile"
     */
    public function enterRenameFile():void {
        focusManager.setFocus(nfname);
    }

    // -------------------------------------------------------------------
    // Event Listeners
    // -------------------------------------------------------------------

    /**
     * A config file has been uploaded
     * @param event
     */
    public function loadEnd(event:LoadEvent):void {
        // rebuild file list with the uploaded file added to the list.
        getFileList();
    }

    // -------------------------------------------------------------------
    // Actions triggered by GUI
    // -------------------------------------------------------------------

    /**
     * REFRESH Button
     */
    public function refresh():void {
        getFileList();
    }

    /**
     * HELP BUTTON
     */
    public function showHelp():void {
        var helpWindow:MonitoringHelp = PopUpManager.createPopUp(this.tabNavigator, MonitoringHelp, true) as MonitoringHelp;
        PopUpManager.centerPopUp(helpWindow);
    }

    /**
     * Upload a Configuration file from the client
     */
    public function uploadConfig():void {
        var loadWindows:LoadWindow = PopUpManager.createPopUp(this.tabNavigator, LoadWindow, true) as LoadWindow;
        var xmlTypes:FileFilter = new FileFilter("XML files", "*.xml");
        loadWindows.fileFilter = new Array(xmlTypes);
        loadWindows.addEventListener(LoadEvent.LOADED, loadEnd);
        PopUpManager.centerPopUp(loadWindows);
    }

    /**
     * A file has been selected from the List of server configuration files
     */
    public function selectFile():void {
        // File is choosen: load it.
        this.cursorManager.setBusyCursor();
        var configfile:ConfigFile = this.listFiles.selectedItem as ConfigFile;
        this.configFileName = configfile.name;
        var path:String = configfile.directory + configfile.separator + configFileName;
        configFileLoad(path);
    }

    public function applyThisConfig():void {
        applyNow = true;
    }

    /**
     * All params have been set
     */
    public function confirmParams():void {
        // Clean any previous operation
        this.strConfigFile = "";

        // Replace params if needed
        for each (var token:String in this.fileTokens) {
            if (paramValues[token] != null) {
                // replace by the chosen value
                this.strConfigFile += (paramValues[token] as TextInput).text;
            }
            else {
                // do not replace this part
                this.strConfigFile += token;
            }
        }

        this.currentState = "selected";
    }

    /**
     * Cancel params
     */
    public function cancelParams():void {
        this.currentState = "";
    }

    /**
     * "Apply" BUTTON
     * Apply this config file: A new dashboard can be created.
     */
    public function applyConfig():void {
        // Create the MonitorWindow for this configuration
        var curConfig:DashboardView = new DashboardView();

        // Label is the basename of the config file (without extension)
        var len:int = configFileName.lastIndexOf(".xml");
        curConfig.label = configFileName.substring(0, len);

        curConfig.init(this.strConfigFile);

        // reset state to initial state
        this.currentState = "pickone";

        // add it to the tabNavigator
        this.tabNavigator.addChild(curConfig);
        this.tabNavigator.selectedIndex = this.tabNavigator.numChildren - 1;

    }

    /**
     * "Edit" BUTTON
     * Popup an Editor to view or Edit source of the config file.
     */
    public function editSource():void {
        // Create Editor in a popup window
        editConfigWindow = PopUpManager.createPopUp(this.configWindow, EditConfigWindow, true) as EditConfigWindow;
        editConfigWindow.width = this.configWindow.width;
        editConfigWindow.height = this.configWindow.height;
        editConfigWindow.title = configFileName;
        editConfigWindow.init(this);
        PopUpManager.centerPopUp(editConfigWindow);
    }

    /**
     * "Save As" BUTTON
     */
    public function saveConfig():void {
        this.currentState = "savefile";
    }

    /**
     * "Rename as" BUTTON
     */
    public function renameConfig():void {
        this.currentState = "renamefile";
    }

    /**
     * Remove BUTTON
     * remove this Configuration file
     */
    public function removeConfig(eventObj:CloseEvent):void {
        if(eventObj.detail == Alert.OK) {
            var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.REMOVE_CONF_EVENT + model.componentID);
            ev.configFile = this.listFiles.selectedItem as ConfigFile;
            dispatcher.dispatchEvent(ev);
        }
    }

    /**
     * Save BUTTON: The name has been set.
     * Actually save the current configuration file
     */
    public function saveFile():void {
        saveConfigFile(this.fname.text);
        this.currentState = "";
    }

    /**
     * Save BUTTON from the Editor
     */
    public function saveChanges():void {
        saveConfigFile(this.configFileName);
    }

    /**
     * Rename BUTTON: The name has been set.
     */
    public function renameFile():void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.RENAME_CONF_EVENT + model.componentID);
        ev.configFile = this.listFiles.selectedItem as ConfigFile;
        ev.loadFile = this.nfname.text;
        dispatcher.dispatchEvent(ev);

        this.currentState = "";
    }

    /**
     * DASHBOARD REMOVE
     * Remove a Tab from the tabNavigator
     * @param name
     */
    public function removeTab(dashboard:DashboardView):void {
        this.tabNavigator.removeChild(dashboard);
    }

    // -----------------------------------------------------------------------------
    // Private Methods
    // -----------------------------------------------------------------------------

    /**
     * Save the ConfigFile with the name specified
     * @param name
     */
    private function saveConfigFile(name:String):void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.SAVE_CONF_EVENT + model.componentID);
        ev.loadFile = name;
        ev.args = this.strConfigFile;
        dispatcher.dispatchEvent(ev);
    }

    /**
     * A file has been selected in the List:
     * Load the file from server.
     * When loaded, strconf() here after will be triggered.
     * @param path
     */
    private function configFileLoad(path:String):void {
        var ev:MonitoringEvent = new MonitoringEvent(MonitoringEvent.LOAD_CONF_EVENT + model.componentID);
        ev.loadFile = path;
        dispatcher.dispatchEvent(ev);
    }

    /**
     * Read the file as a String, and look for params.
     * params should have the form : @paramName@
     * @param file The String form of the config xml file.
     *
     * Build a list of token in this.fileTokens
     * Build a list of params in this.params
     */
    private function lookForParams(file:String):void {
        this.params = null;
        this.fileTokens = new ArrayCollection();
        var tokens:Array = file.split("@");
        var paramfound:Boolean = false;
        for each (var token:String in tokens) {
            this.fileTokens.addItem(token);
            if (paramfound) {
                if (token.length < 22) {
                    // init with token value.
                    // will be filled later by the user.
                    if (this.params == null) {
                        this.params = new ArrayCollection();
                    }
                    if (!this.params.contains(token)) {
                        this.params.addItem(token);
                    }
                }
                else {
                    // Cannot go ahead. Forget parameters
                    Alert.show(locale("config_too_long_param") + token);
                    this.params = null;
                    this.fileTokens = new ArrayCollection();
                    this.fileTokens.addItem(file);
                    break;
                }
            }
            paramfound = !paramfound;
        }
    }


    /**
     * Get the list of configuration files
     */
    private function getFileList():void {
        var ev:MonitoringEvent;
        ev = new MonitoringEvent(MonitoringEvent.GET_CONFIG_FILENAMES_EVENT + model.componentID);
        CairngormEventDispatcher.getInstance().dispatchEvent(ev);
    }

    // -----------------------------------------------------------------------------
    // Progress Bar
    // -----------------------------------------------------------------------------

    /**
     * Title window containing the progress Bar.
     */
    private var titleWindow:TitleWindow = null;
    private var progressBar:ProgressBar = null;

    /**
     * Show a progress bar.
     */
    private function showProgressBar(text:String):void {
        hideProgressBar();

        // create the progress bar.
        progressBar = new ProgressBar();
        progressBar.indeterminate = true;
        progressBar.percentWidth = 100;
        progressBar.label = text;

        // create the popup.
        titleWindow = PopUpManager.createPopUp(this.configWindow, TitleWindow, true) as TitleWindow;
        titleWindow.title = locale("wait");
        titleWindow.width = this.width / 4;
        // add the progress bar to the popup
        titleWindow.addChild(progressBar);
        // center the popup.
        PopUpManager.centerPopUp(titleWindow);
    }

    /**
     * Hide the progress Bar.
     */
    private function hideProgressBar():void {
        if (titleWindow != null) {
            // remove the popup from the screen.
            PopUpManager.removePopUp(titleWindow);
            titleWindow = null;
            progressBar = null;
        }
    }

    // -----------------------------------------------------------------------------
    // Internationalization
    // -----------------------------------------------------------------------------

    /**
     * Get localized String
     */
    public function locale(s:String):String {
        return ResourceManager.getInstance().getString("monitoringResources", s);
    }


}
}
